# FIAIF is an Intelligent firewall, version: $Revision: 1.7 $
#
# description: Automates a packet filtering firewall with iptables.
#
# Script Author: Anders Fugmann <afu at fugmann dot dhs dot org>
# 
# FIAIF is an Intelligent firewall
# Copyright (C) 2002-2003 Anders Peter Fugmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Please drop me a line and give me your thoughts on this software.
# Patches are more than welcome.

###############################################################################
# Delete unreferenced chains.
# Returns: 0 for no work done, 1 othervice.
###############################################################################
function delete_unreferenced_chains
{
    local RETURN=0
    local TABLE=$1
    local CHAIN
    for CHAIN in $(IPTABLES -t ${TABLE} -L -n | \
	grep "(0 references)" | cut -d" " -f 2); do
      IPTABLES -t ${TABLE} -F ${CHAIN}
      IPTABLES -t ${TABLE} -X ${CHAIN}
      RETURN=1
    done
    return ${RETURN}
}


###############################################################################
# List all empty chains. 
# Arguments: TABLE
###############################################################################
function list_empty_chains ()
{
    local TABLE=$1
    local LINE_NO=-1
    declare -i LINE_NO
    local FILE=$(mktemp /tmp/fiaif-tmp.XXXXXX)
    local CHAIN
    local BUILT_IN_CHAINS=BUILT_IN_CHAINS_${TABLE}
    export BUILT_IN_CHAINS=${!BUILT_IN_CHAINS}
    
    (IPTABLES -t ${TABLE} -nL; echo -e "\nChain\n") | while read -a line ; do 
	if [[ ${line[0]} == Chain ]]; then
	    if (( LINE_NO == 1 )); then
		if [[ "${BUILT_IN_CHAINS}" != *${CHAIN}* ]]; then
		    echo ${CHAIN} >> ${FILE}
		fi
	    fi
	    CHAIN=${line[1]}
	    LINE_NO=-1
	else
	    let LINE_NO++
	fi
    done
    RESULT=$(<${FILE})
    rm -f ${FILE}
}

###############################################################################
# Delete references to all chains.
# Arguments: <table> <chain>*
###############################################################################
function delete_chain_references ()
{
    local TABLE=$1
    shift 
    local DEL_CHAINS=$@  
    local line
    local LINE_NO
    local DEL_CHAIN
    local THIS_CHAIN
    declare -i LINE_NO
    
    IPTABLES -t ${TABLE} -nL | while read -a line ; do 
	if [[ ${line[0]} == Chain ]]; then
	    CHAIN=${line[1]}
	    LINE_NO=-1
	else
	    let LINE_NO++
	    if (( LINE_NO > 0 )); then
		for THIS_CHAIN in ${DEL_CHAINS}; do
		    if [[ ${THIS_CHAIN} == ${line[0]} ]]; then
			IPTABLES -t ${TABLE} -D ${CHAIN} ${LINE_NO}
			let LINE_NO--;
		    fi
		done
	    fi
	fi
    done
    
}

###############################################################################
# Main function. Delete all references to empty chains, and
# Delete unreferenced chains.
###############################################################################
function cleanup_rules ()
{
    local CHANGES
    declare -i CHANGES
    local TABLE
    for TABLE in ${TABLES}; do
	CHANGES=1
	while (( CHANGES == 1 )); do 
	    list_empty_chains ${TABLE}
	    if [[ -n "${RESULT}" ]]; then
		delete_chain_references ${TABLE} ${RESULT} 
	    fi
	    delete_unreferenced_chains ${TABLE}
	    CHANGES=$?
	done
    done	
}
